@@ -71,7 +71,7 @@ static ssize_t __sysfs_attr_get_value(struct udev_device *dev, const char *attr_
 	/* read attribute value */
 	fd = open(devpath, O_RDONLY);
 	if (fd < 0) {
-		condlog(3, "%s: attribute '%s' can not be opened: %s",
+		condlog(3, "%s: attribute '%s' cannot be opened: %s",
 			__func__, devpath, strerror(errno));
 		return -errno;
 	}
@@ -137,7 +137,7 @@ ssize_t sysfs_attr_set_value(struct udev_device *dev, const char *attr_name,
 	/* write attribute value */
 	fd = open(devpath, O_WRONLY);
 	if (fd < 0) {
-		condlog(3, "%s: attribute '%s' can not be opened: %s",
+		condlog(3, "%s: attribute '%s' cannot be opened: %s",
 			__func__, devpath, strerror(errno));
 		return -errno;
 	}
@@ -339,3 +339,22 @@ bool sysfs_is_multipathed(struct path *pp, bool set_wwid)
 
 	return found;
 }
+
+struct udev_device *get_udev_for_mpp(const struct multipath *mpp)
+{
+	dev_t devnum;
+	struct udev_device *udd;
+
+	if (!mpp || !has_dm_info(mpp)) {
+		condlog(1, "%s called with empty mpp", __func__);
+		return NULL;
+	}
+
+	devnum = makedev(mpp->dmi.major, mpp->dmi.minor);
+	udd = udev_device_new_from_devnum(udev, 'b', devnum);
+	if (!udd) {
+		condlog(1, "failed to get udev device for %s", mpp->alias);
+		return NULL;
+	}
+	return udd;
+}
